#! /usr/bin/perl -w

use warnings;
use strict;
use Getopt::Std;


our ($opt_s, $opt_f, $opt_l);
my $filename;
my $size;
my %sizes;
my $length;
my $chr;
my $chr_size;
my $line;
my $start;
my $end;
my $count;
my $end_place;

#get command line options

getopt("fsl");
getopts("fsl:");


#if insufficient options, print usage

if(! (($opt_s)&&($opt_f)&&($opt_l)))
	{
	print "\n\nUsage: $0\n\n";
	print "\tREQUIRED:\n";
	print "\t-f name and full path of the positive strand BED file\n";
	print "\t-s name and full path of the chrom.sizes file\n";
	print "\t-l length of tag\n";
	die "\n\n";
	}


#set paramaters
$filename=$opt_f;
$size=$opt_s;
$length=$opt_l;


#open the chromsome sizes file and read the sizes into memory
open(IN, $size) or die "Can't open $size because $!\n";
while($line=<IN>)
	{
	chomp($line);
	($chr, $chr_size)=split(/\t/, $line);
	$sizes{$chr}=$chr_size;
	#convert to zero based coords
	$sizes{$chr}--;
	}
close(IN);
 

#open the positive BED and output WIG files
open(IN, $filename) or die "Can't open $filename because $!\n";
$filename=~s/\.positive//g;
$filename=~s/unique-starts/coverage/g;
$filename=~s/BED/Wig.temp/;
open(WIG, ">$filename") or die "Can't open $filename because $!\n";


my $first='true';
my $curr_chr='';
my %hash;

$end_place='false';

while($line=<IN>)
	{
	chomp($line);
	($chr, $start, $end, $count)=split(/\t/, $line);

	if ($end_place ne 'false') 
		{
		#print out every coordinate that will no longer get modified, and then delete it from the hash to save memory. 
		for (my $i=$end_place; $i<$start; $i++)
			{
			if (exists($hash{$i}))
				{
				my $coord_end=$i+1;
				my $cov=int((($hash{$i}/$length)*100)+0.5);
				print WIG "$curr_chr\t$i\t$coord_end\t$cov\n";
				delete($hash{$i});
				}
			}
		}

	if (($chr ne $curr_chr)&&($first eq 'true'))
		{
		$curr_chr=$chr;
		$first='false';	
		}
		elsif (($chr ne $curr_chr)&&($first ne 'true'))
		{
		print STDOUT "Finished $curr_chr, next will be $chr\n";
		#stop processing and print out current chromosomes data - cuts down on memory usage
		foreach my $coord (sort {$a<=>$b} keys %hash) 
			{
			my $coord_end=$coord+1;
			my $cov=int((($hash{$coord}/$length)*100)+0.5);
			print WIG "$curr_chr\t$coord\t$coord_end\t$cov\n";
			}
		$curr_chr=$chr;	
		undef(%hash);
		my %hash;
		$end_place='false';
		}

	#add the tag data to the wiggle plot
	for (my $i=$start; $i<$end; $i++)
		{
		#make sure that this doesn't fall off the end of the chromosome
		my $stop=$i+$length;
		if ($stop>$sizes{$chr}) {$stop=$sizes{$chr}};
		for (my $j=$i; $j<$stop; $j++)
			{
			$hash{$j}++;
			}
		}

	$end_place=$start;
	}

#print out the last chromosome data
foreach my $coord (sort {$a<=>$b} keys %hash)
	{
	my $coord_end=$coord+1;
	my $cov=int((($hash{$coord}/$length)*100)+0.5);
	print WIG "$curr_chr\t$coord\t$coord_end\t$cov\n";
	}

close(IN);
close(WIG);


#now compress the file 
open(IN, "$filename") or die "Can't open $filename because $!\n";
$filename=~s/\.temp//g;
open(OUT, ">$filename") or die "Can't open $filename because $!\n";

my $first_line=<IN>;
chomp($first_line);
(my $pre_chr, my $pre_start, my $pre_end, my $pre_count) = split (/\t/, $first_line);

while ($line=<IN>)
	{
	chomp($line);
	($chr, $start, $end, $count)= split(/\t/, $line);
	if (($chr eq $pre_chr)&&($start eq $pre_end)&&($count eq $pre_count))
		{
		#merge current results with previous results
		$pre_end=$end;
		}
		else
		{
		#print out prior results and reassign variables to the current variables
		print OUT "$pre_chr\t$pre_start\t$pre_end\t$pre_count\n";
		$pre_chr=$chr;
		$pre_start=$start;
		$pre_end=$end;
		$pre_count=$count;
		}
	}
print OUT "$pre_chr\t$pre_start\t$pre_end\t$pre_count\n";
close(IN);
close(OUT);

unlink("$filename.temp");

exit(0);
