#! /usr/bin/perl -w

use warnings;
use strict;
use Getopt::Std;


our ($opt_s, $opt_f, $opt_l);
my $filename;
my $size;
my %sizes;
my $length;
my $chr;
my $chr_size;
my $line;
my $start;
my $end;
my $count;

#get command line options

getopt("fsl");
getopts("fsl:");


#if insufficient options, print usage

if(! (($opt_s)&&($opt_f)&&($opt_l)))
	{
	print "\n\nUsage: $0\n\n";
	print "\tREQUIRED:\n";
	print "\t-f name and full path of the positive strand BED file\n";
	print "\t-s name and full path of the chrom.sizes file\n";
	print "\t-l length of tag\n";
	die "\n\n";
	}


#set paramaters
$filename=$opt_f;
$size=$opt_s;
$length=$opt_l;


#open the chromsome sizes file and read the sizes into memory
open(IN, $size) or die "Can't open $size because $!\n";
while($line=<IN>)
	{
	chomp($line);
	($chr, $chr_size)=split(/\t/, $line);
	$sizes{$chr}=$chr_size;
	#convert to zero based coords
	$sizes{$chr}--;
	}
close(IN);
 

#open the positive and negative files
open(IN, $filename) or die "Can't open $filename because $!\n";
$filename=~s/positive/negative/g;
open(BED, ">$filename") or die "Can't open $filename because $!\n";


while($line=<IN>)
	{
	chomp($line);
	($chr, $start, $end, $count)=split(/\t/, $line);
	$start=$start+$length;
	$end=$end+$length;

	#make sure that the coords don't fall off the end of the chromosome
	if ($end>$sizes{$chr}) {$end=$sizes{$chr}};
	
	#print out to BED file
	if ($start>$sizes{$chr}) {} else {print BED "$chr\t$start\t$end\t$count\n";}
	}
close(IN);
close(BED);

exit(0);
